package com.ashokit.ticketbooking.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

@Configuration
@EnableWebSecurity
//@EnableWebSecurity loads SecurityConfig. java as default security filter and it will load on startup
public class SecurityConfig {
	
	
	
	//Authenitcation Logic
	
	/*
	 * Need to create 2 users 1) ashokitadmin/ashokitadmin ==> ADMIN Role
	 *                        2) ashokitguest/ashokitguest ==> GUEST Role
	 */
	
	
	
	
	
	//UserDetailsService ==> Services Class which is used to  create authentication object
	//UserDetails ==> Spring Framework Authentication Object
	
	
	/*
	 * 2.x Code
	
	
	
	
	public void configure(AuthenticationManagerBuilder auth) throws Exception {

		
		auth.inMemoryAuthentication().user("ashokitadmin").password("{noop}ashokitadmin").role("admin")
		//.and().formLogin()/httpbasci();
	}
	
	 */
	@Bean
	public UserDetailsService userDetailsService(PasswordEncoder encoder) {
		
		
		UserDetails admin=User.withUsername("ashokitadmin").
				password(encoder.encode("ashokitadmin")).roles("ADMIN").build();
		
		
		
		UserDetails guest=User.withUsername("ashokitguest")
				.password(encoder.encode("ashokitguest")).roles("GUEST").build();
		
		return new InMemoryUserDetailsManager(admin,guest);
		
	}
	
	@Bean
	public PasswordEncoder passwordEncoder() {
		//BCryptPasswordEncoder ==> Base64 Encoder
		return new BCryptPasswordEncoder();
	}
	
	//Authorizatio Logic
	
	
	
	
	
	@Bean
	public SecurityFilterChain securityFilterChain(HttpSecurity authObj) throws Exception {
		//SecurityFilterChain ==> It will contain all authorization rules
		
		
		//2.X Code
		//formLogin ==> It will prompt Login page
		
		
		/*
		authObj.csrf().disable() //Next class
		.authorizeHttpRequests().requestMatchers("/tickets/all/**").hasAnyRole("ADMIN")
		.and().formLogin(); 
		*/
		
		
		//Admin Role can Access All Tickets API
		/*authObj.csrf(csrf -> csrf.disable())
		.authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/all/**"))
				.hasAnyRole("ADMIN")).formLogin();*/
		
		
		
		authObj.csrf(csrf -> csrf.disable())
		.authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/all/**"))
				.hasAnyRole("ADMIN")).httpBasic();
		
		
		//Guest Role Can access Individual Ticket API
		/*authObj.csrf(csrf -> csrf.disable())
		.authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/ticket/**"))
				.hasAnyRole("GUEST")).formLogin();*/
		
		
		authObj.csrf(csrf -> csrf.disable())
		.authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/ticket/**"))
				.hasAnyRole("GUEST","ADMIN")).httpBasic();
		
		return authObj.build();
		
	}

}
